// Violations 17/4/17 - JR
// Variation - Colossus/Core Workshop
// TRIGGER set to +5V
// pic18f26k22
// MPLAB X IDE v3.55
// Note - all PWM generators initialized

#include <htc.h>
#include <stdlib.h>
#include <pic18f26k22.h>

// PIC18F26K22 Configuration Bit Settings

// CONFIG1H
#pragma config FOSC = INTIO67   // Oscillator Selection bits (Internal oscillator block)
#pragma config PLLCFG = OFF     // 4X PLL Enable (Oscillator used directly)
#pragma config PRICLKEN = ON    // Primary clock enable bit (Primary clock enabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRTEN = OFF     // Power-up Timer Enable bit (Power up timer disabled)
#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
#pragma config BORV = 190       // Brown Out Reset Voltage bits (VBOR set to 1.90 V nominal)

// CONFIG2H
#pragma config WDTEN = OFF//ON       // Watchdog Timer Enable bits (WDT is always enabled. SWDTEN bit has no effect)
//#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC1  // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF//ON      // PORTB A/D Enable bit (PORTB pins are configured as analog input channels on Reset)
#pragma config CCP3MX = PORTB5  // P3A/CCP3 Mux bit (P3A/CCP3 input/output is multiplexed with RB5)
#pragma config HFOFST = ON      // HFINTOSC Fast Start-up (HFINTOSC output and ready status are not delayed by the oscillator stable status)
#pragma config T3CMX = PORTC0   // Timer3 Clock input mux bit (T3CKI is on RC0)
#pragma config P2BMX = PORTB5   // ECCP2 B output mux bit (P2B is on RB5)
#pragma config MCLRE = EXTMCLR  // MCLR Pin Enable bit (MCLR pin enabled, RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection Block 0 (Block 0 (000800-003FFFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection Block 1 (Block 1 (004000-007FFFh) not code-protected)
#pragma config CP2 = OFF        // Code Protection Block 2 (Block 2 (008000-00BFFFh) not code-protected)
#pragma config CP3 = OFF        // Code Protection Block 3 (Block 3 (00C000-00FFFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection Block 0 (Block 0 (000800-003FFFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection Block 1 (Block 1 (004000-007FFFh) not write-protected)
#pragma config WRT2 = OFF       // Write Protection Block 2 (Block 2 (008000-00BFFFh) not write-protected)
#pragma config WRT3 = OFF       // Write Protection Block 3 (Block 3 (00C000-00FFFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection Block 0 (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection Block 1 (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection Block 2 (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection Block 3 (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)

#define _XTAL_FREQ 16000000
#define SWITCH3 PORTAbits.RA5 // user sequence input switch (RA5 = pin 7)
#define TRIGGER PORTAbits.RA7 // CV sequence trigger - index 0

/*
 * Implements a delay of a number of milliseconds. 
 */
void delay(int ms) {
    while (ms-- > 0) {
        __delay_ms(1);
    }
}

////GLOBAL VALUE
int inc = 0; // increment value - in ISR
int dec = 0; // decrement value - in ISR
int count = 0; // counter for timer in ISR

// User input arrays
char user_input[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // user generated sequence
char current_index = 0;
char array_length = 0; // index for user_input array

/*
 * Initialises the variable resistor ADC configuration. Variable resistor for tempo/speed of sequence.
 */
void initADC() {
    // 1. Configure Port:
    //  - Disable pin output driver(See TRIS register)
    TRISAbits.RA0 = 1;

    //  - Configure pin as analog
    ANSELAbits.ANSA0 = 1;

    // 2. Configure the ADC module:
    //  - Select ADC conversion clock
    ADCON2bits.ADCS = 0b101; // Fosc/16

    //  - Configure voltage reference
    ADCON1bits.PVCFG = 0; // +ref is VDD
    ADCON1bits.NVCFG = 0; // -ref is VSS

    //  - Select ADC input channel
    ADCON0bits.CHS = 0; // connect AN0 to the ADC

    //  - Select result format
    ADCON2bits.ADFM = 0; // left justified

    //  - Select acquisition delay
    ADCON2bits.ACQT = 0b101; // 12 TAD

    //  - Turn on ADC module
    ADCON0bits.ADON = 1;
}

/*
 * Reads the value of the variable resistors into a single byte.
 *
 */
unsigned int readADC(unsigned char channel) {
    unsigned int AN_Val;

    // Select desired channel for ADC operation
    switch (channel) {
        case 1: // AN0
            ADCON0bits.CHS = 0;
            break;
        case 2: // AN3
            ADCON0bits.CHS = 3;
            break;
        default: // Any other value will default to AN0
            ADCON0bits.CHS = 0; // AN0
    } // switch (channel)

    // Channel selected proceed with ADC conversion
    __delay_us(10); // sampling time
    // start a conversion
    ADCON0bits.GO = 1; //This sets the go/!done bit that starts conversion. Bit will be cleared when ADC is complete

    while (ADCON0bits.GO == 1)
        __delay_us(10); // longer delay here influences the ISR and timers

    AN_Val = ADRESH; //returns 8-bit?

    return AN_Val; // int ADC(unsigned char channel)
}

/*
 * Initialises PWM on the three PWM generators.
 */
void initPWM() {
    // See PIC18F26K22 data sheet p. 186
    // 1. Disable the CCPx pin output driver by setting the associated TRIS bit.
    TRISC = 0b00000110; // RC1 (PWM2) and RC2 (PWM1)
    TRISBbits.RB5 = 1; //PWM3 pin 26
    TRISBbits.RB0 = 1; //PWM4 pin 21
    TRISAbits.RA4 = 1; //PWM5 pin 6

    // 2. Select the 8-bit TimerX resource, (Timer2, Timer4 or Timer6) to be
    //    used for PWM generation by setting the CxTSEL bits in the
    //    CCPTMRSx register.
    CCPTMRS0bits.C1TSEL = 0; // user timer 2

    // 3. Load the PRx register for the selected TimerX with the PWM period value.
    PR2 = 0x40; // decimal 64 period

    // 4. Configure the CCP module for the PWM mode by loading the CCPxCON
    //    register with the appropriate values.
    CCP1CONbits.CCP1M = 0b1100;
    CCP2CONbits.CCP2M = 0b1100;
    CCP3CONbits.CCP3M = 0b1100;
    CCP4CONbits.CCP4M = 0b1100;
    CCP5CONbits.CCP5M = 0b1100;

    // 5. Load the CCPRxL register and the DCxB bits of the CCPxCON register,
    //    with the PWM duty cycle value.
    CCPR1L = 128 >> 2;
    CCPR2L = 128 >> 2;
    CCPR3L = 128 >> 2;
    CCPR4L = 128 >> 2;
    CCPR5L = 128 >> 2;

    CCP1CONbits.DC1B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP2CONbits.DC2B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP3CONbits.DC3B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP4CONbits.DC4B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP5CONbits.DC5B = 128 & 0b00000011; // 128 is 50% duty cycle

    // 6. Configure and start the 8-bit TimerX resource:
    //    - Clear the TMRxIF interrupt flag bit of the
    //      PIR2 or PIR4 register. See Note 1 below.
    PIR1bits.TMR2IF = 0;

    //    - Configure the TxCKPS bits of the TxCON
    //      register with the Timer prescale value.
    T2CONbits.T2CKPS = 0b01; // prescale 1:4 p. 177. This is not 8(0b11)!

    // - Enable the Timer by setting the TMRxON
    // bit of the TxCON register.
    T2CONbits.TMR2ON = 1;

    // 7. Enable PWM output pin:
    //    - Wait until the Timer overflows and the TMRxIF bit of the PIR2 or PIR4
    //      register is set. See Note 1 below.
    while (PIR1bits.TMR2IF == 0) {
    }

    //    - Enable the CCPx pin output driver by clearing the associated TRIS bit.
    TRISC = 0; // RC1 (PWM2) and RC2 (PWM1)
    TRISBbits.RB5 = 0; //PWM3
    TRISBbits.RB0 = 0; //PWM4
    TRISAbits.RA4 = 0; //PWM5
}

/*
 * Sets the PWM values for each of the PWM controllers.
 */
void setPWM(char value1) {
    CCPR1L = value1 >> 2;
    CCP1CONbits.DC1B = value1 & 0b00000011;

//    CCPR2L = value2 >> 2;
//    CCP2CONbits.DC2B = value2 & 0b00000011;
}


// interrupt service routine
void interrupt ISR(void) {

//    if (ctr2 == 1)
        user_input[array_length] = readADC(2); // read_analogue_value channel (case) 2

//    else
//        if (ctr2 == 2)
//        user_input2[array_length2] = readADC(2); // read_analogue_value channel (case) 2

    // has the interrupt been fired by timer0?
    if (TMR0IF == 1) {
        count++;

        if (count >= (255 / (readADC(1) + 1) * 2)) // yeesssss!
        {

            current_index++; // user_input1 forward
//            current_index2++; // user_input1 backward
            
            // reset counter
            count = 0;
        }
        // reset the overflow interrupt flag
        TMR0IF = 0;
    }
}

void init_timer0(void) {

    T0CON = 0x00; //00000000 - stop timer; 16-bit; internal; source edge; pre-scaler select; pre-scale values
    //Timer will runfrom Internal instruction cycle clock , Prescalar will be used

    T0CONbits.T08BIT = 1; //USING Timer0 as 16-bit counter 1 = 8-bit; 0 = 16-bit
    //T0CONbits.PSA = 0; //set to pre-scaler
    T0CONbits.T0PS = 0b110; // pre-scale
    T0CONbits.TMR0ON = 1; //Turned ON timer0

    INTCONbits.TMR0IE = 1; //Enabling Timer0 Interrupt

}

void play() {
        setPWM(user_input[current_index]);
}

void main() {
    // set up the internal oscillator
    OSCCONbits.IRCF = 0b111; // 16MHz

    // up down tactile switches
    TRISAbits.RA1 = 1; // RA1 pin 3 as input SW1 (up) - 0 output/1 input
    TRISAbits.RA2 = 1; // RA2 pin 4 as input SW2 (down)
    TRISAbits.RA5 = 1; // RA5 pin 7 user sequencer input
    TRISAbits.RA6 = 1; // RA5 pin 10 user sequencer input 2

    // CV trigger input
    TRISAbits.RA7 = 1; // RA7 pin 9 as trigger input for reset table
    //TRISBbits.RB1 = 1; //test delete - input

    ANSELAbits.ANSA1 = 0; //  - Configure pin as analog 1 digital 0
    ANSELAbits.ANSA2 = 0; //  - Configure pin as analog 1 digital 0
    ANSELAbits.ANSA5 = 0; //  - Configure pin as analog 1 digital 0

    TRISC = 0;
    TRISB = 0; //needs to be selected

    // interrupt settings
    INTCONbits.GIE = 1; // Global Interrupt Enable

    // init the PWM
    initPWM();

    // init the ADC
    initADC();

    // initialize timer
    init_timer0();

    while (1) {
        if (TRIGGER == 0) {
            play();
        }

        if (TRIGGER == 1) // if the gate/trigger is on
        {
            // reset the sequence
            inc = 0;
            dec = 0;
            current_index = 0;
            count = 0; // reset ISR counter
        }

        if (SWITCH3 == 0) {
            delay(40); // debounce .. holding SWITCH3 for user_input live
            if (SWITCH3 == 0)// if still pressed
                array_length++; // now increment to the next space in the array
            delay(40); // debounce
            if (array_length >= 16)
                array_length = 0; // if we reach the end of the array, go back to the start
        }
        if (current_index > array_length) {
            current_index = 0;
        }
    }
}